<?php declare(strict_types=1);
namespace MyTestNamespace;
require __DIR__ . '/bootstrap.php';
require __DIR__ . '/testClasses.php';
use function Grifart\AssertFunction\{assertSignature, nullable, params};
use Grifart\AssertFunction\FunctionSignatureAssertionError;
use Tester\Assert;
$place = 'tests/' . basename(__FILE__) . ':' . (__LINE__ +2) . ' ';

$fn = function(?T1 $t1, ?T2 $t2): ?T3 {return new T3;};

// Correct signature assertion
assertSignature($fn, params(nullable(T1::class), nullable(T2::class)), nullable(T3::class));
assertSignature($fn, params(nullable(T1::class), nullable(T2::class)), NULL); // no expectation about return type

// parameter is nullable, non-nullable is expected
Assert::exception(
	function () use ($fn) { // 1st param
		assertSignature($fn, params(T1::class, nullable(T2::class)), nullable(T3::class));
	},
	FunctionSignatureAssertionError::class,
	$place . 'Parameter #1 ($t1) is expected to be required. But it is nullable.'
);

Assert::exception(
	function () use ($fn) { // 2nd param
		assertSignature($fn, params(nullable(T1::class), T2::class), nullable(T3::class));
	},
	FunctionSignatureAssertionError::class,
	$place . 'Parameter #2 ($t2) is expected to be required. But it is nullable.'
);

Assert::exception(
	function () use ($fn) { // return type
		assertSignature($fn, params(nullable(T1::class), nullable(T2::class)), T3::class);
	},
	FunctionSignatureAssertionError::class,
	$place . 'Return-type is expected to be required. But it is nullable.'
);

// Wrong return type
Assert::exception(
	function () use ($fn) {
		assertSignature($fn, params(nullable(T1::class), nullable(T2::class)), nullable(T2::class));
	},
	FunctionSignatureAssertionError::class,
	$place . "Expected return type of type '" . T2::class . "', "
		. "but given function declares '" . T3::class . "'."
);

// Wrong parameter type
Assert::exception(
	function () use ($fn) {
		assertSignature($fn, params(nullable(T1::class), nullable(T1::class)), nullable(T3::class));
	},
	FunctionSignatureAssertionError::class,
	$place . 'Given function parameter #2 $t2 has type ' . T2::class . '. '
		. 'However this parameter is expected to be ' . T1::class . '.'
);
