<?php declare(strict_types=1);
namespace MyTestNamespace;
require __DIR__ . '/bootstrap.php';
require __DIR__ . '/testClasses.php';
use function Grifart\AssertFunction\{assertSignature, nullable, params};
use Grifart\AssertFunction\FunctionSignatureAssertionError;
use Tester\Assert;

$place = 'tests/' . basename(__FILE__) . ':' . (__LINE__ +2) . ' ';

$f1 = function(T1 $t1, T2 $t2): T3 {return new T3;};

// Positive scenario
assertSignature($f1, params(T1::class, T2::class), T3::class);
assertSignature($f1, [T1::class, T2::class], T3::class); // alternative shorter less-self explaining syntax

// Requires optional parameter, function has required
Assert::exception(
	function () use ($f1) {
		assertSignature($f1, params(nullable(T1::class), T2::class), T3::class);
	},
	FunctionSignatureAssertionError::class,
	$place . 'Parameter #1 ($t1) is expected to be nullable. '
		. 'But it is required.'
);

// Requires optional return type, function has required
Assert::exception(
	function () use ($f1) {
		assertSignature($f1, params(T1::class, T2::class), nullable(T3::class));
	},
	FunctionSignatureAssertionError::class,
	$place . 'Return-type is expected to be nullable. '
		. 'But it is required.'
);
